package entity

import (
	"bytes"
	"fmt"
	"github.com/gin-gonic/gin"
	"gitlab.wanzhuangkj.com/tush/xpkg/gin/ctxUtils"
)

type CopyHttpReq struct {
	Method  string              `json:"method"`
	URL     string              `json:"url"`
	Headers map[string][]string `json:"headers"`
	Body    string              `json:"body"`
}

type CopyHttpRsp struct {
	Headers map[string][]string `json:"headers"`
	Body    string              `json:"body"`
}

func SetCopyReq(c *gin.Context, buf *bytes.Buffer) {
	copyHttpReq := &CopyHttpReq{}
	copyHttpReq.Method = c.Request.Method
	copyHttpReq.URL = c.Request.URL.String()
	copyHttpReq.Headers = c.Request.Header
	copyHttpReq.Body = buf.String()
	c.Set(ctxUtils.KeyReqBody, copyHttpReq)
}

func GetCopyReq(c *gin.Context) *CopyHttpReq {
	if val, isExist := c.Get(ctxUtils.KeyReqBody); isExist {
		if req, ok := val.(*CopyHttpReq); ok {
			return req
		}
	}
	return nil
}

func SetCopyRsp(c *gin.Context, buf *bytes.Buffer) {
	copyHttpRsp := &CopyHttpRsp{}
	copyHttpRsp.Headers = c.Writer.Header()
	copyHttpRsp.Body = buf.String()
	c.Set(ctxUtils.KeyRspBody, copyHttpRsp)
}

func GetCopyRsp(c *gin.Context) *CopyHttpRsp {
	if val, isExist := c.Get(ctxUtils.KeyRspBody); isExist {
		if rsp, ok := val.(*CopyHttpRsp); ok {
			return rsp
		}
	}
	return nil
}

func SetCopyApiReq(c *gin.Context, buf *bytes.Buffer) {
	copyHttpReq := &CopyHttpReq{}
	copyHttpReq.Method = c.Request.Method
	copyHttpReq.URL = c.Request.URL.String()
	copyHttpReq.Headers = c.Request.Header
	copyHttpReq.Body = buf.String()
	c.Set(fmt.Sprintf(ctxUtils.KeyOApiReqBody, ctxUtils.GetGinCtxTraceID(c)), copyHttpReq)
}

func GetCopyApiReq(c *gin.Context) *CopyHttpReq {
	if val, isExist := c.Get(fmt.Sprintf(ctxUtils.KeyOApiReqBody, ctxUtils.GetGinCtxTraceID(c))); isExist {
		if req, ok := val.(*CopyHttpReq); ok {
			return req
		}
	}
	return nil
}

func SetCopyApiRsp(c *gin.Context, buf *bytes.Buffer) {
	copyHttpRsp := &CopyHttpRsp{}
	copyHttpRsp.Headers = c.Writer.Header()
	copyHttpRsp.Body = buf.String()
	c.Set(fmt.Sprintf(ctxUtils.KeyOApiRspBody, ctxUtils.GetGinCtxTraceID(c)), copyHttpRsp)
}

func GetCopyApiRsp(c *gin.Context) *CopyHttpRsp {
	if val, isExist := c.Get(fmt.Sprintf(ctxUtils.KeyOApiRspBody, ctxUtils.GetGinCtxTraceID(c))); isExist {
		if rsp, ok := val.(*CopyHttpRsp); ok {
			return rsp
		}
	}
	return nil
}
