package xsf

import (
	"fmt"
	"strconv"

	"github.com/bwmarrin/snowflake"
	"gitlab.wanzhuangkj.com/tush/xpkg/xerrors/xerror"
)

var node *snowflake.Node

type ID snowflake.ID

func (f ID) MarshalJSON() ([]byte, error) {
	buff := make([]byte, 0, 22)
	buff = append(buff, '"')
	buff = strconv.AppendInt(buff, int64(f), 10)
	buff = append(buff, '"')
	return buff, nil
}

func (f *ID) UnmarshalJSON(b []byte) error {
	if len(b) == 0 {
		return nil
	}
	if len(b) == 1 {
		i, err := strconv.ParseInt(string(b), 10, 64)
		if err != nil {
			return err
		}
		*f = ID(i)
		return nil
	}
	if len(b) == 2 {
		if b[0] == '"' && b[1] == '"' {
			*f = ID(0)
		}
		i, err := strconv.ParseInt(string(b), 10, 64)
		if err != nil {
			return err
		}
		*f = ID(i)
		return nil
	}
	if b[0] == '"' && b[len(b)-1] == '"' {
		i, err := strconv.ParseInt(string(b[1:len(b)-1]), 10, 64)
		if err != nil {
			return err
		}
		*f = ID(i)
		return nil
	}
	i, err := strconv.ParseInt(string(b), 10, 64)
	if err != nil {
		return err
	}
	*f = ID(i)
	return nil
}

type JSONSyntaxError struct{ original []byte }

func (j JSONSyntaxError) Error() string {
	return fmt.Sprintf("invalid xsf ID %q", string(j.original))
}

// n [1,1023]
func Init(n int64) (err error) {
	node, err = snowflake.NewNode(n)
	if err != nil {
		return err
	}
	return nil
}

func Generate() ID {
	return ID(node.Generate())
}

func (x ID) String() string {
	return snowflake.ID(x).String()
}

func (x ID) ToInt64() int64 {
	return snowflake.ID(x).Int64()
}

func (x ID) ToInt() int {
	return int(snowflake.ID(x).Int64())
}

func GenerateID() ID {
	sid := Generate()
	return ID(sid)
}

func ParseInt64(val int64) ID {
	return ID(snowflake.ParseInt64(val))
}

func ParseString(val string) (ID, error) {
	v, err := snowflake.ParseString(val)
	if err != nil {
		return 0, xerror.New(err.Error())
	}
	return ID(v), nil
}
